/*********************************************************
 *                                                       *
 * Dessert Roadmap                                       *
 *                                                       *
 * This is actually a classic Mandelbrot set picture :)  *
 *                                                       *
 * Created by Brian Vinter, June 15th 1999               *
 * Modified to generic graphics class Febuary 12 2001    *
 *                                                       *
 * This is not an example of perfect Java code,          *
 * but is meant as a simple basis for generating         *
 * parallel versions.                                    *
 ********************************************************/


import java.lang.Math;
import java.awt.*;

class colorTable{  
  Color[] colortable = {Color.white, //We use this to translate
				Color.white,         //between iterations and
				Color.blue,          //colors.
				Color.cyan, 
				Color.black, 
				Color.green, 
				Color.magenta, 
				Color.orange, 
				Color.pink, 
				Color.red, 
				Color.yellow 
};

  static int color_cnt=10;           //Highest index in the colortable
}
 
public class RoadMap                                //This is our map generator
{
  static double box_x_min, box_x_max,               //This is the bounding box
                box_y_min, box_y_max;               //of the map

  static graphicsScreen g;

  //These two functions translates between pixels and bounding box
  static double translate_x(int x) {       
    return (((box_x_max-box_x_min)/g.width)*x)+box_x_min;
  }

  static double translate_y(int y) {
    return (((box_y_max-box_y_min)/g.hight)*y)+box_y_min;
  }
  
  
  static int solve(double x, double y)              //Simple Mandelbrot
  {                                          //divergation test
    double r=0.0,s=0.0;
    double next_r,next_s;
    int itt=0;
    
    while((r*r+s*s)<=4.0) {
      next_r=r*r-s*s+x;
      next_s=2*r*s+y;
      r=next_r; s=next_s;
      if(++itt==100)break;
    }
    
    return itt;
  }
  

  static public void CreateMap() {        //Our 'main' function

    int color;                               //Holds the result of slove
	colorTable ct = new colorTable();

    for(int y=0;y<g.hight;y++)              //Main loop for map generation
      for(int x=0;x<g.width;x++){  
	    color=solve(translate_x(x),translate_y(y))*ct.color_cnt/100;
	    g.plot(x,y,ct.colortable[color]);                 //Plot the coordinate to map
      }
  }


  public static void main(String [] args) {
    g = new graphicsScreen(500,500);
    StopWatch time = new StopWatch();
    double start, stop;
    int zooms=100;

	ProgressBar pb = new ProgressBar();       //Just to allow us to monitor progress;)
    pb.setMaximum(zooms);
	
    box_x_min=-1.5; box_x_max=0.5;           //Set the map bounding box for total map
    box_y_min=-1.0; box_y_max=1.0;

	double deltaxmin=(-0.9-box_x_min)/zooms;
	double deltaxmax=(-0.65-box_x_max)/zooms;
	double deltaymin=(-0.4-box_y_min)/zooms;
	double deltaymax=(-0.1-box_y_max)/zooms;

    time.start();                    //Get starttime
    for(int i=0;i<zooms;i++){
      CreateMap();                     //Call our main
  	  box_x_min+=deltaxmin;
	  box_x_max+=deltaxmax;
	  box_y_min+=deltaymin;
	  box_y_max+=deltaymax;
	  pb.Inc();
    }                       
    time.stop();
    System.out.println("Time taken: "+time.timeString());
  }
}
